from flask import Flask, render_template, request
import csv

app = Flask(__name__)

#Code cesar
cesar = {'A': 'C', 'B': 'D', 'C': 'E', 'D': 'F', 'E': 'G', 'F': 'H', 'G': 'I', 'H': 'J', 'I': 'K', 'J': 'L', 
         'K': 'M', 'L': 'N', 'M': 'O', 'N': 'P', 'O': 'Q', 'P': 'R', 'Q': 'S', 'R': 'T', 'S': 'U', 'T': 'V',
         'U': 'W', 'V': 'X', 'W': 'Y', 'X': 'Z', 'Y': 'A', 'Z': 'B', 'a': 'c', 'b': 'd', 'c': 'e', 'd': 'f', 'e': 'g', 'f': 'h', 'g': 'i', 'h': 'j', 'i': 'k', 'j': 'l', 
         'k': 'm', 'l': 'n', 'm': 'o', 'n': 'p', 'o': 'q', 'p': 'r', 'q': 's', 'r': 't', 's': 'u', 't': 'v',
         'u': 'w', 'v': 'x', 'w': 'y', 'x': 'z', 'y': 'a', 'z': 'b', '0': '2', '1': '3', '2': '4', '3': '5',
         '4': '6', '5': '7', '6': '8', '7': '9', '8': '0', '9': '1'
}

#Fonction qui va crypter le mot passé en paramètres avec un code Cesar
def encode_mot(mot, cesar):
    message = ""
    for char in mot:
        if char in cesar:
            message += cesar[char]
        else:
            message += char
                
    return message

#Affiche le menu
@app.route('/')
def index():
    return render_template('index.html')

#Affiche la page de création d'un compte
@app.route('/create_account')
def create_account():
    return render_template('create_account.html')

#Affiche la page de connexion à un compte existant
@app.route('/login_account')
def login_account():
    return render_template('login_account.html')

#Page pour créer un compte (seulement si l'username n'est pas déjà pris)
@app.route('/create_traite', methods=['POST'])
def create_traite():
    #Récupère username et password
    username = request.form['username']
    password = request.form['password']
    
    #Si aucun username ou password n'a été rentré on retourne une erreur
    if username == "" or password == "":
        return "[ERREUR] : Le nom d'utilisateur ou le mot de passe entré est vide."
    
    #On ouvre le fichier usernames en mode lecture
    with open("system/usernames.csv", 'r') as file:
        #On enregistre son contenu dans une variable
        tableau_users = list(csv.reader(file,delimiter=";"))
        #Si l'username est déjà pris alors on retourne une erreur
        if username in tableau_users:
            return "[ERREUR] : Ce nom d'utilisateur est déjà pris, veuillez en choisir un autre."
        
    #On ouvre le fichier usernames en mode écriture sans supprimer le contenu précédent
    with open("system/usernames.csv", 'a', newline='') as file:
        #On écrit le nouveau nom d'utilisateur dans le fichier usernames
        obj = csv.writer(file)
        obj.writerow(username.split(" "))
        
    #On encode le mot de passe
    password_crypt = encode_mot(password, cesar)
    
    #On ouvre le fichier csv login_infos en mode écriture ("+" pour créer le fichier)
    with open(f"system/login_infos-{username}.csv", 'w+') as file:
        #On écrite toutes les infos dans le fichier
        file.write(f"{username}\n{password_crypt}")
        
    #On affiche que le compte a bien été créé et propose d'aller sur la page de login
    return render_template('create_account_success.html')

#Page pour se connecter à un compte utilisateur existant
@app.route('/login_traite', methods=['POST'])
def login_traite():
    #Récupère username et password
    username = request.form['username']
    password = request.form['password']
    
    #Si aucun username ou password n'a été rentré on retourne une erreur
    if username == "" or password == "":
        return "[ERREUR] : Le nom d'utilisateur ou le mot de passe entré est vide."
    
    #On ouvre le fichier usernames en mode lecture
    with open("system/usernames.csv", 'r') as file:
        #On enregistre son contenu dans une variable
        tableau_users = list(csv.reader(file,delimiter=";"))
        #On transforme le tableau de tableau en tableau simple pour la suite
        for nb,x in enumerate(tableau_users):    
            tableau_users[nb] = ''.join(x)

        #Si l'username est bien présent dans le fichier alors :
        if username in tableau_users:
            #On encode le mot de passe
            password_crypt = encode_mot(password, cesar)
            #On ouvre le fichier de données de l'utilisateur en mode lecture
            with open(f"system/login_infos-{username}.csv", 'r') as file:
                donnees = list(csv.reader(file,delimiter=";"))
                #On transforme le tableau de tableau en tableau simple pour la suite
                for nb,x in enumerate(donnees):    
                    donnees[nb] = ''.join(x)
                    
                #Si le mot de passe correspond avec celui enregistré alors:
                if password_crypt == donnees[1]:
                    #On affiche que l'utilisateur s'est bien connecté
                    return render_template('sujets_index.html')
                #Sinon on affiche une erreur
                else:
                    return f"[ERREUR] : Le mot de passe \"{password}\" n'est pas valide."
        #Sinon on affiche une erreur
        else:
            return f"[ERREUR] : Le nom d'utilisateur \"{username}\" n'est pas valide."

#Affiche la page pour créer un sujet
@app.route('/create')
def create_topics():
    return render_template('create_topic.html')

@app.route('/create_topics_traite', methods=['POST'])
def create_topics_traite():
    #Dictionnaire construit d’après les valeurs des attributs name et value d’un formulaire.
    values = request.form['sujet']
    #Ouvre le fichier où les sujets sont stockés
    if values != "":
        with open("system/sujets.csv", 'a', newline='') as file:
            #On écrit le nouveau sujet dans le fichier sujets.csv
            obj = csv.writer(file)
            obj.writerow(values.split(" "))
    else:
        return "Veuillez rentrer un nom de sujet non vide"
        
    #Retourne le menu du compte
    return render_template('sujets_index.html')

#Affiche la page avec la liste des sujets
@app.route('/look')
def show_topics():
    #Ouvre le fichier où les sujets sont stockés
    with open("system/sujets.csv", "r") as file:
        #Lit le fichier
        sujets = csv.reader(file, delimiter=";")
        #Retourne la page html avec les fichiers
        return render_template('show_topics.html', liste=sujets)



app.run(debug=True)

























